/*
 * Copyright (c) 2003 MIPS Technologies, Inc.  All rights reserved.
 * 
 * Unpublished rights (if any) reserved under the copyright laws of the
 * United States of America and other countries.
 * 
 * This code is proprietary to MIPS Technologies, Inc. ("MIPS
 * Technologies").  Any copying, reproducing, modifying or use of this
 * code (in whole or in part) that is not expressly permitted in writing
 * by MIPS Technologies or an authorized third party is strictly
 * prohibited.  At a minimum, this code is protected under unfair
 * competition and copyright laws.  Violations thereof may result in
 * criminal penalties and fines.
 * 
 * MIPS Technologies reserves the right to change this code to improve
 * function, design or otherwise.  MIPS Technologies does not assume any
 * liability arising out of the application or use of this code, or of
 * any error or omission in such code.  Any warranties, whether express,
 * statutory, implied or otherwise, including but not limited to the
 * implied warranties of merchantability or fitness for a particular
 * purpose, are excluded.  Except as expressly provided in any written
 * license agreement from MIPS Technologies or an authorized third party,
 * the furnishing of this code does not give recipient any license to any
 * intellectual property rights, including any patent rights, that cover
 * this code.
 * 
 * This code shall not be exported or transferred for the purpose of
 * reexporting in violation of any U.S. or non-U.S. regulation, treaty,
 * Executive Order, law, statute, amendment or supplement thereto.
 * 
 * This code constitutes one or more of the following: commercial
 * computer software, commercial computer software documentation or other
 * commercial items.  If the user of this code, or any related
 * documentation of any kind, including related technical data or
 * manuals, is an agency, department, or other entity of the United
 * States government ("Government"), the use, duplication, reproduction,
 * release, modification, disclosure, or transfer of this code, or any
 * related documentation of any kind, is restricted in accordance with
 * Federal Acquisition Regulation 12.212 for civilian agencies and
 * Defense Federal Acquisition Regulation Supplement 227.7202 for
 * military agencies.  The use of this code by the Government is further
 * restricted in accordance with the terms of the license agreement(s)
 * and/or applicable contract terms and conditions covering this code
 * from MIPS Technologies or an authorized third party.
 */

/*
 * ex2/tlbxcpt.c: MIPS SDE example #2 (exception handling)
 */


/*
 * Example #2
 * Demonstrates use of xception handler;
 * and TLB functionality, uncached access only.
 * Tests basic operation of kuseg and kseg2 and TLB reload.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>
#include <mips/xcpt.h>
#include <mips/cpu.h>

/* NOT POSIX !! */
extern void    *sbrk (int);

static int	ntlb;
static int	nwired;

#define NITER	500

#if ! #mmu(no)

#if #mmu(type2)
#define TLBPGSIZE (VMPGSIZE * 2)	/* each tlb entry maps two pages */
#elif #mmu(type1) || #mmu(cw10)
#define TLBPGSIZE VMPGSIZE		/* each tlb entry maps one page */
#else
#error unknown mmu architecture
#endif

#define TLBPGMASK (TLBPGSIZE - 1)

static int      ntlbmiss;
static unsigned char *tpg;

/*
 * This function is called on a TLBmiss exception
 */
static int
mytlbhandler (int code, struct xcptcontext *xcp)
{
    unsigned        pfn = paToPn (KVA_TO_PA (tpg));
    reg_t           va = xcp->vaddr;
    static reg_t    lastva = ~0;

#ifdef VERBOSE
    fprintf (stderr, "\nTLB reload va = %08x\n", REG_TO_VA (va));
#endif

    if (va == lastva) {
	fprintf (stderr, "\nrepeated TLB miss va = %08x\n", REG_TO_VA (va));
	return SIGBUS;
    }
    lastva = va;

#if #mmu(type1)
    mips_tlbwr (mktlbhi (vaToVpn (va), 0),
		mktlblo (pfn, TLB_V | TLB_G | TLB_N));
#elif #mmu(type2)
    mips_tlbwr2 (mktlbhi (vaToVpn (va), 0),
		 mktlblo (pfn, TLB_V | TLB_G | TLB_UNCACHED),
		 mktlblo (pfn + 1, TLB_V | TLB_G | TLB_UNCACHED),
		 0);
#elif #mmu(cw10)
    cw401x_tlbwr (mktlbhi (vaToVpn (va), 0),
		  mktlblo (pfn, TLB_V | TLB_G | TLB_UNCACHED), 
		  0);
#else
#error unhandled mmu architecture
#endif
    ntlbmiss++;
    return 0;
}


static void
checkbrand0 (void *base, unsigned range, int niter)
{
    int             iter;

    for (iter = 0; iter < niter; iter++) {
	unsigned        exp, got;
	unsigned        r = rand () * rand ();

	r %= range;

	if ((iter % 64) == 0)
	    fprintf (stderr, "Pass %8d\r", iter);

        exp = tpg[r & TLBPGMASK];
	got = ((unsigned char *)base)[r];
        if (got != exp) {
	    fprintf (stderr, "Pass %d error expected %#x at %08x, but got %#x\n",
		     iter, exp, base + r, got);
	    abort ();
	}
    }
}
#endif /* !#mmu(no) */


static void
clear_tlb (void)
{
#if !#mmu(no)
    unsigned long pfn = (unsigned long)KSEG1_BASE >> VMPGSHIFT;
    unsigned int sr = _mips_intdisable ();
    int i;

#if #mmu(type1)
    for (i = nwired; i < ntlb; i++)
	mips_tlbwi (mktlbhi (pfn + (ntlb - 1 - i), 0), 0, i);
#elif #mmu(type2)
    for (i = nwired; i < ntlb; i++)
	mips_tlbwi2 (mktlbhi (pfn + (ntlb - 1 - i) * 2, 0), 0, 0, 0x0, i);
#elif #mmu(cw10)
    for (i = nwired; i < ntlb; i++)
	cw401x_tlbwi (mktlbhi (pfn + (ntlb - 1 - i), 0), 0, 0x0, i);
#else
#error unhandled mmu architecture
#endif
    _mips_intrestore (sr);
#endif
}


static void
done (int status)
{
    clear_tlb ();

    exit (status);
}


int
main (int argc, char **argv, char **envp)
{
    unsigned        pfn;
    unsigned 	    sr;
    int             niter = NITER;
    int		    i;

    ntlb = mips_tlb_size ();
    if (ntlb <= 0) {
	fprintf (stderr, "No TLB on this CPU, sorry!\n");
	exit (EXIT_FAILURE);
    }

#if #mmu(no)

    fprintf (stderr, "TLB size %d, but no TLB support!\n", ntlb);
    exit (EXIT_FAILURE);

#else

    /* use unwired tlb entries only */
    nwired = mips_getwired ();

    printf ("TLB size: %d entries, %d wired\n", ntlb, nwired);
    
    /* align current break to page size */
    {
	unsigned va = (unsigned)sbrk (0);
	if (va & TLBPGMASK)
	    (void)sbrk (TLBPGSIZE - (va & TLBPGMASK));
    }

    /* allocate a physical page and fill with 
     * arbitrary values (the code for main!)
     */
    tpg = sbrk (TLBPGSIZE);
    assert (((unsigned)tpg & TLBPGMASK) == 0);

    /* fill with a distinctive pattern */
    for (i = 0; i < TLBPGSIZE; i++)
	tpg[i] = i;

    /* clean writeback cache */
    if (IS_KVA0 (tpg))
	mips_clean_dcache ((vaddr_t)tpg, TLBPGSIZE);

    /* reset TLB */
    clear_tlb ();

    /* tlb[0] := va=0,tlbid=0,global,readonly,uncached */
    pfn = KVA_TO_PA (tpg) >> VMPGSHIFT;		/* phys page number */

    /* set first unwired entry */
    sr = _mips_intdisable ();			/* disable interrupts */
#if #mmu(type1)
    mips_tlbwi (mktlbhi (0, 0),
		mktlblo (pfn, TLB_V | TLB_G | TLB_N),
		nwired);
#elif #mmu(type2)
    mips_tlbwi2 (mktlbhi (0, 0),
		mktlblo (pfn, TLB_V | TLB_G | TLB_UNCACHED),
		mktlblo (pfn + 1, TLB_V | TLB_G | TLB_UNCACHED),
		0x0, nwired);
#elif #mmu(cw10)
    cw401x_tlbwi (mktlbhi (0, 0),
		  mktlblo (pfn, TLB_V | TLB_G | TLB_UNCACHED),
		  0x0, nwired);
#else
#error unhandled mmu architecture
#endif
    _mips_intrestore (sr);			/* restore interrupts */

    /* check contents match */
    if (memcmp ((void *)0, tpg, TLBPGSIZE) != 0) {
	fprintf (stderr, "KUSEG failed virtual to physical equivalence\n");
	done (EXIT_FAILURE);
    }

    /* reset TLB again */
    clear_tlb ();

    /* trap read TLB faults with a handler which reloads to zpg */
    (void)xcption (XCPTTLBL, mytlbhandler);

    /* run over half KUSEG randomly accesing memory over the whole range */
    ntlbmiss = 0;
    {
	void           *base = KUSEG_BASE;
	unsigned        range = KUSEG_SIZE / 2;

	printf ("KUSEG tlb reload test: base %08x range %08x\n", base, range);
	checkbrand0 (base, range, niter);
	printf ("Done %8d iterations: %d tlbmisses\n", niter, ntlbmiss);
    }

    /* run over KUSEG but restrict to a small range 
     * an R3000-type TLB contains 64 entries each mapping 4K 
     * an R4000-type TLB contains 16-48 entries each mapping 8K
     * of these the first NWIRED are not written by tlbwr 
     */
    ntlbmiss = 0;
    {
	void           *base = KUSEG_BASE;
	unsigned        range = (ntlb - nwired) * TLBPGSIZE / 2; /* half range */

	printf ("KUSEG tlb reload test: base %08x range %08x\n", base, range);
	checkbrand0 (base, range, niter);
	printf ("Done %8d iterations: %d tlbmisses\n", niter, ntlbmiss);
    }

    ntlbmiss = 0;
    {
	void           *base = KSEG2_BASE;
	unsigned        range = KSEG2_SIZE;

	printf ("KSEG2 tlb reload test: base %08x range %08x\n", base, range);
	checkbrand0 (base, range, niter);
	printf ("Done %8d iterations: %d tlbmisses\n", niter, ntlbmiss);
    }

    done (EXIT_SUCCESS);
#endif /* !#mmu(no) */
}
