# Copyright (c) 1996-2003 MIPS Technologies, Inc.  All rights reserved.
# 
# Unpublished rights (if any) reserved under the copyright laws of the
# United States of America and other countries.
# 
# This code is proprietary to MIPS Technologies, Inc. ("MIPS
# Technologies").  Any copying, reproducing, modifying or use of this
# code (in whole or in part) that is not expressly permitted in writing
# by MIPS Technologies or an authorized third party is strictly
# prohibited.  At a minimum, this code is protected under unfair
# competition and copyright laws.  Violations thereof may result in
# criminal penalties and fines.
# 
# MIPS Technologies reserves the right to change this code to improve
# function, design or otherwise.  MIPS Technologies does not assume any
# liability arising out of the application or use of this code, or of
# any error or omission in such code.  Any warranties, whether express,
# statutory, implied or otherwise, including but not limited to the
# implied warranties of merchantability or fitness for a particular
# purpose, are excluded.  Except as expressly provided in any written
# license agreement from MIPS Technologies or an authorized third party,
# the furnishing of this code does not give recipient any license to any
# intellectual property rights, including any patent rights, that cover
# this code.
# 
# This code shall not be exported or transferred for the purpose of
# reexporting in violation of any U.S. or non-U.S. regulation, treaty,
# Executive Order, law, statute, amendment or supplement thereto.
# 
# This code constitutes one or more of the following: commercial
# computer software, commercial computer software documentation or other
# commercial items.  If the user of this code, or any related
# documentation of any kind, including related technical data or
# manuals, is an agency, department, or other entity of the United
# States government ("Government"), the use, duplication, reproduction,
# release, modification, disclosure, or transfer of this code, or any
# related documentation of any kind, is restricted in accordance with
# Federal Acquisition Regulation 12.212 for civilian agencies and
# Defense Federal Acquisition Regulation Supplement 227.7202 for
# military agencies.  The use of this code by the Government is further
# restricted in accordance with the terms of the license agreement(s)
# and/or applicable contract terms and conditions covering this code
# from MIPS Technologies or an authorized third party.


#
# kit/kit.mk
#
# Shared make include file for MIPS SDE embedded system kit


.PHONY: all buildall

all:	buildall

# defaults, which may be overridden by sbd.mk
BAREMON	=bare
CPU	=r3k
CACHES	=$(CP0)
TLB	=$(CP0)
WPOINTS =$(CACHES)
NVENV	=noenv
FPU	=no
PCI	=no
ENDIAN	=-EB
ROM_U	=bfc00000
ROM_C	=9fc00000
ROM_SZ	=400000
RAM_U	=a0000280
RAM_C	=80000280

INCDIRS	=

XCPTLOW =xcptlowb

ifndef FREESDE
-include ../fullsde.mk
endif

KITDIR	=..

# Here we include the board-specific sbd.mk
include sbd.mk

# Pick Cop0 support files - defaulting to "m32" for known MIPS32 CPUs and ISAs
M32CPUs	=4k% 5k% 20k% 24k% 25k% m4k%
M32ISAs	=-mips32% -mips64%

ifndef CP0
 ifneq (,$(filter $(M32CPUs),$(CPU))$(filter $(M32ISAs),$(ISA)))
  CP0	=m32
 else
  CP0	=$(CPU)
 endif
endif

ifndef ROMDLFMT
  ROMDLFMT=s3
endif

KITALL	=ramstart.o sastart.o libram.a librom.a crtn.o

ifdef SBDLOW
KITALL	+=romstart.o micromon.$(ROMDLFMT)
endif

ALL	=kit

LIB_TEST=kit-test
APP_TEST=example-test

kit:	$(KITALL)

ifndef CACHEDEF
  CACHEDEF=$(foreach cache,$(CACHES),'-Acache($(cache))')
endif

ifndef CACHESRC
  CACHESRC=cache.sx $(foreach cache,$(CACHES),$(cache)cache.sx)
  CACHEOBJ=cache.o $(foreach cache,$(CACHES),$(cache)cache.o)
  CACHEXOBJ=cachex.o $(foreach cache,$(CACHES),$(cache)cachex.o)
endif

ifndef WATCHDEF
  WATCHDEF=$(foreach wp,$(WPOINTS),'-Awatch($(wp))')
endif

ifndef WATCHSRC
  # XXX should do this like caches above to reduce library size
  # XXX but we haven't updated the individual sbd.mk files yet
  # WATCHSRC=$(foreach wp,$(WPOINTS),$(wp)watch.c)
  # WATCHOBJ=$(foreach wp,$(WPOINTS),$(wp)watch.o)
  WATCHSRC=r4kwatch.c r5kwatch.c r4650watch.c rm7kwatch.c \
	   lr33watch.c cw4watch.c m32watch.c r3kwatch.c
  WATCHOBJ=r4kwatch.o r5kwatch.o r4650watch.o rm7kwatch.o \
	   lr33watch.o cw4watch.o m32watch.o r3kwatch.o
endif

ifdef FULLSDE
%cachex.o: %cache.sx
	$(COMPILE.S) -D_SDE_CACHE_EXTRA $< -o $@

cachex.o: cache.sx
	$(COMPILE.S) -D_SDE_CACHE_EXTRA $< -o $@
endif

ifndef CP1
 ifeq ($(FPU),no)
  CP1		=noc1
 else
  CP1		=$(CP0)c1
 endif
endif

# Set up search path for sources only
_VPATH	=$(SBDDIRS):../$(MONITOR):../$(BAREMON):../share:../pci

##
## Generic kit source files
##

KITSRC	=romlow.sx $(XCPTLOW).sx $(TLB)tlb.sx $(CP1).sx $(CACHESRC) \
	 xcpt.c xcptshow.c xcptstack.c intrupt.c cp1emu.c \
	 nsdelay.c \
	 watch.c watchsup.c $(WATCHSRC) \
	 stubs.c posix.c dirops.c getdir.c mfs.c sysconf.c \
	 signals.c profil.c mon.c \
	 m16entry.c muldivem.c bremu.c unaligned.c \
	 mipscp0.sx mips16.sx clz.c \
	 fcache.c flashdev.c paneldev.c env.c $(NVENV).c

##
## Generic kit object files
##

# These objects are startup code which is "position-independent", and
# uses "bal" instead "jal" instructions, so they must be within 128K of 
# each other.
KITLOW	=romlow.o $(TLB)tlb.o $(CACHEOBJ)

KITOBJ	=xcpt.o xcptshow.o xcptstack.o $(XCPTLOW).o cp1emu.o $(CP1).o \
	 nsdelay.o \
	 watch.o watchsup.o $(WATCHOBJ) \
	 stubs.o posix.o dirops.o getdir.o mfs.o sysconf.o \
	 signals.o ecchandler.o \
	 intrupt.o timer.o profil.o mon.o \
	 $(CACHEOBJ) $(CACHEXOBJ) $(TLB)tlb.o \
	 m16entry.o muldivem.o bremu.o unaligned.o \
	 mipscp0.o mips16.o clz.o \
	 fcache.o flashdev.o paneldev.o env.o $(NVENV).o

# List of optional generic features supported in this version of the kit.
KITFEATURES =xcptstackinfo

ifneq ($(PCI),no)
  KITSRC += pciconf.c pcidevs.c pciio.c pcilookup.c
  KITOBJ += pciconf.o pcidevs.o pciio.o pcilookup.o
  SBDFEATURES += pci pcilookup
endif

# monitor interfaces for ROM and RAM
include ../$(BAREMON)/monitor.mk
include ../$(MONITOR)/monitor.mk

# Include some per-architecure specific files and settings
-include $(KITDIR)/archs.mk

ALLOBJS =$(sort $(SBDLOW) $(SBDOBJ) \
	   $(KITLOW) $(KITOBJ) \
	   $(MONLOW) $(MONOBJ) \
	   $(BARELOW) $(BAREOBJ))

ifdef SBDLOW
ALLOBJS +=micromon.o
endif

# Compute list of object files generated from board-specific source files
#LOCALOBJS=$(filter %.o,$(patsubst %.c,%.o,$(patsubst %.sx,%.o,$(SBDSRC))))

# We no longer supply any board-specific source code in free/eval version
LOCALOBJS=

# Then compute the list of all "shared" objects (i.e. ones not derived
# from board-specific source code).
SHAREDOBJS=$(filter-out $(LOCALOBJS),$(ALLOBJS))

CPUFLAGS=-mcpu=$(CPU) $(ISA) $(ENDIAN)

INCDIRS+=-I. $(patsubst %,-I%,$(subst :, ,$(SBDDIRS)))
ifneq ($(PCI),no)
INCDIRS+=-I../pci
endif
INCDIRS+=-I../$(MONITOR) -I../share -I- $(STDINCDIR)

# define DEBUG=-UNDEBUG to enable assertions in library
ifndef DEBUG
DEBUG	=-DNDEBUG 
endif

ifndef FULLSDE
FREEKIT=-DFREE_SDE
endif

CPPFLAGS=$(SBDFLAGS) $(SBDOPTIONS) $(KITFLAGS) $(DEBUG) $(CPUVARIANT) $(CACHEDEF) '-Ammu($(TLB))' $(ARCHFLAGS) $(INCDIRS) $(FREEKIT)

ifneq (,$(strip $(filter -m%,$(FPU))))
CPUFLAGS+=$(FPU)
CPPFLAGS+='-Afpu($(patsubst -m%,%,$(FPU)))'
else
CPPFLAGS+='-Afpu($(FPU))'
endif

# Add board-specific features
CPPFLAGS+=$(foreach feature,$(SBDFEATURES) $(MONFEATURES),'-Afeature($(feature))')

ifneq ($(PCI),no)
 CPPFLAGS+= '-Afeature(pciconf)' '-Apci($(PCI))'
endif

ifneq ($(ROM_U),bfc00000)
 CPPFLAGS+=-DROM_BASE=0x$(ROM_U)
endif
ifneq ($(RAM_U),a0000280)
 CPPFLAGS+=-DRAM_BASE=0x$(RAM_U)
endif

# User may override these options
ifndef FREEBUILD
override CFLAGS	=-Os -g $(KITCFLAGS)
override ASFLAGS=-O -g -Werror
else
# No source line info in free libs - what's the point
override CFLAGS	=-Os -g1 $(KITCFLAGS)
override ASFLAGS=-O -Werror
endif

override WFLAGS	=-Wall -Wno-parentheses -Wno-uninitialized -Wmissing-prototypes -Winline -Werror

# For Algor assembler uncomment this line
#ASFLAGS	+=-Wa,-algas,-V

# But not these
override CFLAGS	+=-mno-gpopt -fno-optimize-sibling-calls $(WFLAGS) $(CPUFLAGS)
override ASFLAGS+=-G0 $(CPUFLAGS)
override LDFLAGS+=$(ENDIAN)

ifdef LIBRARY_PATH
 # split LIBRARY_PATH into separate -L flags for linker
 override LDFLAGS += $(patsubst %,-L%,$(subst :, ,$(LIBRARY_PATH)))
endif

# These objects are startup code which is "position-independent", and
# uses "bal" instead "jal" instructions, so they must be within 128K of 
# each other.

# modules that MUST come first in a ROM program
romstart.o: $(KITLOW) $(SBDLOW) $(BARELOW)
	$(LINK) $(LDFLAGS) -r -o $@ $^

# modules that MUST come first in a downloadable program
ramstart.o: $(MONLOW)
	$(LINK) $(LDFLAGS) -r -o $@ $^

# modules that MUST come first in a standalone downloadable program
sastart.o: $(BARELOW)
	$(LINK) $(LDFLAGS) -r -o $@ $^

librom.a: $(KITOBJ) $(SBDOBJ) $(BAREOBJ)
	$(AR) $(ARFLAGS) $@ $?

libram.a: $(KITOBJ) $(SBDOBJ) $(MONOBJ)
	$(AR) $(ARFLAGS) $@ $?

crtn.o: crtn.sx

# Select the correct arguments for sde-conv when making a ROM
ifeq ($(ROM_U)/$(ROM_SZ),bfc00000/400000)
  ROMFLG=-p
else
  ROMFLG=-a $(ROM_U)/$(ROM_SZ)
endif

micromon.$(ROMDLFMT): micromon
	sde-conv -vf $(ROMDLFMT) $(ROMFLG) -o micromon.$(ROMDLFMT) micromon

ifdef SBDLOW
micromon: micromon.o $(SBDLOW) $(TLB)tlb.o $(CACHEOBJ)
	$(LINK) $(LDFLAGS) -Ttext $(ROM_U) -Tdata $(RAM_U) -o $@ $^
else
micromon: 
	@echo "Cannot build micromon without SBDLOW defined"
endif

#$(ALLOBJS): Makefile sbd.mk ../kit.mk
#Makefile sbd.mk ../kit.mk:

CLEAN	=kitclean
DEPEND	=kitdepend freedepend

-include ../../rtpx/rtpx.mk

-include ../../net/net.mk

buildall: $(ALL) # reminder

.PHONY: clean test kitclean reallyclean

clean: $(CLEAN)

reallyclean: clean
	rm -f *.pco *~
	rm -f $(LIB_TEST) $(APP_TEST)

.PHONY: test lib-test app-test
test:
	@$(MAKE) SBD=$(SBD) lib-test 
	@$(MAKE) SBD=$(SBD) app-test
	@$(MAKE) SBD=$(SBD) clean

lib-test: $(LIB_TEST)

app-test: $(APP_TEST)

kit-test: 
	@$(MAKE) SBD=$(SBD) kitclean
	@$(MAKE) SBD=$(SBD) kit
	@touch $@

example-test: minimon-test tlbxcpt-test paranoia-test whetstone-test
	@touch $@

.PHONY: minimon-test tlbxcpt-test paranoia-test whetstone-test
minimon-test:
	@$(MAKE) -C ../../examples/minimon SBD=$(SBD) KITBUILT=y

tlbxcpt-test:
	@$(MAKE) -C ../../examples/tlbxcpt SBD=$(SBD) KITBUILT=y

paranoia-test:
	@$(MAKE) -C ../../examples/paranoia SBD=$(SBD) KITBUILT=y

whetstone-test:
	@$(MAKE) -C ../../examples/whetstone SBD=$(SBD) KITBUILT=y

kitclean:
	rm -f micromon micromon.$(ROMDLFMT) *.o *.a core *.core

.PHONY: depend kitdepend freedepend

depend: $(DEPEND)

kitdepend: $(filter-out $(NDASRC),$(KITSRC) $(SBDSRC) $(BARESRC) $(MONSRC))
	@echo Building depend.mk
	@$(CC) $(CFLAGS) $(CPPFLAGS) -MM $? >depend.new
	@if cmp -s depend.mk depend.new; 	\
	 then					\
	   echo depend.mk unchanged;		\
	   rm depend.new;			\
	 else					\
	   rm -f depend.mk;			\
	   mv depend.new depend.mk;		\
	 fi

ifneq (,$(strip $(LOCALOBJS)))
freedepend: $(SBDSRC)
	@echo Building fdepend.mk
	@$(CC) $(CFLAGS) $(CPPFLAGS) -MM $? >fdepend.new
	@if cmp -s fdepend.mk fdepend.new; 	\
	 then					\
	   echo fdepend.mk unchanged;		\
	   rm fdepend.new;			\
	 else					\
	   rm -f fdepend.mk;			\
	   mv fdepend.new fdepend.mk;		\
	 fi
else
freedepend: 
endif

.PHONY: TAGS
TAGS: $(KITSRC) $(SBDSRC) $(BARESRC) $(MONSRC)
	etags $?

vpath	%.c	$(_VPATH)
vpath	%.S	$(_VPATH)
vpath	%.sx	$(_VPATH)
vpath	%.s	$(_VPATH)

PCOLIB	=../free/$(SBD).lib

.PRECIOUS: $(PCOLIB)

.PHONY: reminder

# List of precompiled object files
ifndef FULLSDE

# Extract pre-compiled .o files from pco library
$(SHAREDOBJS): $(PCOLIB)
	$(AR) xv $? $@

$(PCOLIB):
	@if [ ! -e $(PCOLIB) ]; then \
	  echo '********************************************************'; \
	  if [ -e .notfree ]; then \
	    echo "$(SBD) target not supported in the free version of SDE."; \
	    echo "If can only be built with the supported SDE package."; \
	  else \
	    echo "$(SBD) target library not found. Check that you have"; \
	    echo "installed the appropriate target board library package."; \
	  fi; \
	  echo '********************************************************'; \
	  exit 1; \
	fi

# Yes it is intentionally easy to remove this nag message!
reminder:
	@echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
	@echo "+++      Building with MIPS Technologies' SDE lite       +++"
	@echo "+++ For a supported and more configurable product visit  +++"
	@echo "+++        http://www.mips.com/products/sde.html         +++"
	@echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"

ifneq (,$(strip $(LOCALOBJS)))
 -include ./fdepend.mk
endif

else

# Build pco library from .o files
.PHONY: free
free: $(SHAREDOBJS)
	rm -f $(PCOLIB)
	$(AR) $(ARFLAGS) $(PCOLIB) $?
	rm -f $(SHAREDOBJS)

# Extract pre-compiled .o files from pco library
$(NDAOBJ): $(PCOLIB)
	$(AR) xv $? $@

reminder:

# Special assembler dependency
$(XCPTLOW).o: xcptcache.s

-include ./depend.mk

endif

include ../rules.mk
